@@ -2,7 +2,8 @@ import asyncio
 import logging
 import signal
 
-from contextlib import suppress
+from dataclasses import dataclass
+from contextlib import suppress, asynccontextmanager
 from multiprocessing import Process, Event
 from multiprocessing.synchronize import Event as EventType
 
@@ -24,7 +25,7 @@ from .netprotocol import (
     PublisherInfo
 )
 
-from typing import Dict, List, Tuple, Optional
+from typing import Dict, List, Tuple, Optional, AsyncGenerator
 
 logger = logging.getLogger('ezmsg')
 
@@ -50,19 +51,6 @@ class GraphServer(Process):
         self._server_up = Event()
         self._shutdown = Event()
 
-    def shutdown(self) -> None:
-        self._shutdown.set()
-
-    @staticmethod
-    async def open(address: AddressType = GRAPHSERVER_ADDR) -> Tuple[asyncio.StreamReader, asyncio.StreamWriter]:
-        reader, writer = await asyncio.open_connection(*address)
-        writer.write(uint64_to_bytes(getnode()))
-        await writer.drain()
-        server_version = await read_str(reader)
-        if server_version != __version__:
-            raise VersionMismatch(f"GraphServer@{server_version} != Client@{__version__}")
-        return reader, writer
-
     @classmethod
     async def ensure_running(cls, address: AddressType = GRAPHSERVER_ADDR) -> Optional["GraphServer"]:
         graph_server = None
@@ -82,6 +70,10 @@ class GraphServer(Process):
         asyncio.run(self._serve())
         signal.signal(signal.SIGINT, handler)
 
+    def stop(self) -> None:
+        self._shutdown.set()
+        self.join()
+
     async def _serve(self) -> None:
         loop = asyncio.get_running_loop()
         self._command_lock = asyncio.Lock()
@@ -102,6 +94,13 @@ class GraphServer(Process):
             pass
         
         finally:
+
+            for task in self._client_tasks.values():
+                task.cancel()
+                with suppress(asyncio.CancelledError):
+                    await task
+            self._client_tasks.clear()
+            
             monitor_task.cancel()
             with suppress( asyncio.CancelledError ):
                 await monitor_task
@@ -132,7 +131,6 @@ class GraphServer(Process):
             del self.clients[id]
             writer.close()
 
-
     async def api(self, reader: asyncio.StreamReader, writer: asyncio.StreamWriter) -> None:
         try:
             node = await read_int(reader)
@@ -147,6 +145,10 @@ class GraphServer(Process):
             if not req:
                 return
 
+            if req == Command.SHUTDOWN.value:
+                self._shutdown.set()
+                return
+
             # We only want to handle one command at a time
             async with self._command_lock:
 
@@ -185,32 +187,18 @@ class GraphServer(Process):
                     from_topic = await read_str(reader)
                     to_topic = await read_str(reader)
 
-                    if req == Command.CONNECT.value:
-                        try:
-                            self.graph.add_edge(from_topic, to_topic)
-                        except CyclicException:
-                            writer.write(Command.CYCLIC.value)
-                            await writer.drain()
-                            return
+                    cmd = self.graph.add_edge
+                    if req == Command.DISCONNECT.value:
+                        cmd = self.graph.remove_edge
 
-                    elif req == Command.DISCONNECT.value:
-                        self.graph.remove_edge(from_topic, to_topic)
+                    try:
+                        cmd(from_topic, to_topic)
+                        for sub in self._downstream_subs(to_topic):
+                            await self._notify_subscriber(sub)
+                        writer.write(Command.COMPLETE.value)
+                    except CyclicException:
+                        writer.write(Command.CYCLIC.value)
 
-                    for sub in self._downstream_subs(to_topic):
-                        await self._notify_subscriber(sub)
-
-                    writer.write(Command.COMPLETE.value)
-                    await writer.drain()
-
-                elif req == Command.PAUSE.value:
-                    for pub in self._publishers():
-                        try:
-                            async with pub.sync_writer() as pub_writer:
-                                pub_writer.write(Command.PAUSE.value)
-                        except (ConnectionResetError, BrokenPipeError):
-                            continue
-
-                    writer.write(Command.COMPLETE.value)
                     await writer.drain()
 
                 elif req == Command.SYNC.value:
@@ -224,19 +212,16 @@ class GraphServer(Process):
                     writer.write(Command.COMPLETE.value)
                     await writer.drain()
 
+                elif req == Command.PAUSE.value:
+                    for pub in self._publishers():
+                        pub.writer.write(Command.PAUSE.value)
+
                 elif req == Command.RESUME.value:
                     for pub in self._publishers():
-                        try:
-                            async with pub.sync_writer() as pub_writer:
-                                pub_writer.write(Command.RESUME.value)
-                        except (ConnectionResetError, BrokenPipeError):
-                            continue
-
-                    writer.write(Command.COMPLETE.value)
-                    await writer.drain()
+                        pub.writer.write(Command.RESUME.value)
 
                 else:
-                    logger.warn(f'GraphConnection API received unknown command {req}')
+                    logger.warn(f'GraphServer received unknown command {req}')
 
         except (ConnectionResetError, BrokenPipeError):
             logger.debug('GraphServer connection fail mid-command')
@@ -273,41 +258,63 @@ class GraphServer(Process):
         return [sub for sub in self._subscribers() if sub.topic in downstream_topics]
 
     @staticmethod
-    async def connect(from_topic: str, to_topic: str, address: AddressType = GRAPHSERVER_ADDR) -> None:
-        reader, writer = await GraphServer.open(address)
-        writer.write(Command.CONNECT.value)
-        writer.write(encode_str(from_topic))
-        writer.write(encode_str(to_topic))
+    async def open(address: AddressType = GRAPHSERVER_ADDR) -> Tuple[asyncio.StreamReader, asyncio.StreamWriter]:
+        reader, writer = await asyncio.open_connection(*address)
+        writer.write(uint64_to_bytes(getnode()))
         await writer.drain()
-        response = await reader.read(1)
-        if response == Command.CYCLIC.value:
-            raise CyclicException
+        server_version = await read_str(reader)
+        if server_version != __version__:
+            raise VersionMismatch(f"GraphServer@{server_version} != Client@{__version__}")
+        return reader, writer
 
-    @staticmethod
-    async def disconnect(from_topic: str, to_topic: str, address: AddressType = GRAPHSERVER_ADDR) -> None:
-        reader, writer = await GraphServer.open(address)
-        writer.write(Command.DISCONNECT.value)
-        writer.write(encode_str(from_topic))
-        writer.write(encode_str(to_topic))
-        response = await reader.read(1) # Complete
+    @dataclass
+    class Connection:
 
-    @staticmethod
-    async def pause(address: AddressType = GRAPHSERVER_ADDR) -> None:
-        reader, writer = await GraphServer.open(address)
-        writer.write(Command.PAUSE.value)
-        await writer.drain()
-        response = await reader.read(1) # Complete
+        address: Address
 
-    @staticmethod
-    async def sync(timeout: Optional[float] = None, address: AddressType = GRAPHSERVER_ADDR) -> None:
-        reader, writer = await GraphServer.open(address)
-        writer.write(Command.SYNC.value)
-        await writer.drain()
-        response = await asyncio.wait_for(reader.read(1), timeout=timeout) # Complete
+        @asynccontextmanager
+        async def _open(self) -> AsyncGenerator[Tuple[asyncio.StreamReader, asyncio.StreamWriter], None]:
+            reader, writer = await GraphServer.open(self.address)
+            try:
+                yield reader, writer
+            finally:
+                writer.close()
+                await writer.wait_closed()
+
+        async def connect(self, from_topic: str, to_topic: str) -> None:
+            async with self._open() as (reader, writer):
+                writer.write(Command.CONNECT.value)
+                writer.write(encode_str(from_topic))
+                writer.write(encode_str(to_topic))
+                await writer.drain()
+                response = await reader.read(1)
+                if response == Command.CYCLIC.value:
+                    raise CyclicException
+
+        async def disconnect(self, from_topic: str, to_topic: str) -> None:
+            async with self._open() as (reader, writer):
+                writer.write(Command.DISCONNECT.value)
+                writer.write(encode_str(from_topic))
+                writer.write(encode_str(to_topic))
+                await writer.drain()
+                await reader.read(1) # Complete
+
+        async def sync(self, timeout: Optional[float] = None) -> None:
+            async with self._open() as (reader, writer):
+                reader, writer = await GraphServer.open(self.address)
+                writer.write(Command.SYNC.value)
+                await writer.drain()
+                await asyncio.wait_for(reader.read(1), timeout=timeout) # Complete
+
+        async def pause(self) -> None:
+            async with self._open() as (reader, writer):
+                writer.write(Command.PAUSE.value)
+
+        async def resume(self) -> None:
+            async with self._open() as (reader, writer):
+                writer.write(Command.RESUME.value)
+
+        async def shutdown(self) -> None:
+            async with self._open() as (reader, writer):
+                writer.write(Command.SHUTDOWN.value)
 
-    @staticmethod
-    async def resume(address: AddressType = GRAPHSERVER_ADDR) -> None:
-        reader, writer = await GraphServer.open(address)
-        writer.write(Command.RESUME.value)
-        await writer.drain()
-        response = await reader.read(1) # Complete
